#!/bin/sh

if [ "$1" == "" ]
then
    echo "Usage: $0 dev"
    exit 1
fi

DEV="$1"

DC3DD="src/dc3dd"

# clean up temp files from any previous runs
function cleanup()
{
    rm hd_001.* HD_001.log
    if [ -e "hd_001.001" -o -e "HD_001.log" ]
    then
        echo "Failed to remove test output files"
        exit 2
    fi
}

function check_hashes()
{
    m_acq=`grep md5\ TOTAL HD_001.log | awk '{print $3}' | head -1`
    m_join=`grep md5\ TOTAL HD_001.log | awk '{print $3}' | tail -1`

    if [ "$m_acq" != "$m_join" ]
    then
        echo "acquisition/join MD5 hash mismatch"
        exit 3
    fi

    m_ext=`cat hd_001.* | md5sum | awk '{print $1}'`

    if [ "$m_join" != "$m_ext" ]
    then
        echo "cat|dc3dd / cat|md5sum MD5 hash mismatch"
        echo "dc3dd:  $m_join"
        echo "md5sum: $m_ext"
        exit 3
    fi


    s_acq=`grep sha1\ TOTAL HD_001.log | awk '{print $3}' | head -1`
    s_join=`grep sha1\ TOTAL HD_001.log | awk '{print $3}' | tail -1`

    if [ "$s_acq" != "$s_join" ]
    then
        echo "acquisition/join SHA1 hash mismatch"
        exit 3
    fi

    s_ext=`cat hd_001.* | sha1sum | awk '{print $1}'`

    if [ "$s_join" != "$s_ext" ]
    then
        echo "cat|dc3dd / cat|sha1sum SHA1 hash mismatch"
        echo "dc3dd:   $s_join"
        echo "sha1sum: $s_ext"
        exit 3
    fi
}

function join_cat()
{
    cleanup

    "$DC3DD" conv=sync,noerror if="$DEV" hash=md5,sha1 log=HD_001.log split=1900M splitformat=111 of=hd_001 progress=on progresscount=1000 errors=group

    if [ $? -ne 0 ]
    then
        echo "Acquisition Run Failed"
        exit 3
    fi

    cat hd_001.* | "$DC3DD" of=/dev/null hash=md5,sha1 log=HD_001.log progress=on progresscount=1000 hashwindow=1900M

    if [ $? -ne 0 ]
    then
        echo "cat Join Run Failed"
        exit 3
    fi

    check_hashes
}

function join_cat2()
{
    cleanup

    "$DC3DD" conv=sync,noerror if="$DEV" hash=md5,sha1 hashwindow=1900M log=HD_001.log split=1900M splitformat=111 of=hd_001 progress=on progresscount=1000 errors=group

    if [ $? -ne 0 ]
    then
        echo "Acquisition Run Failed"
        exit 3
    fi

    cat hd_001.* | "$DC3DD" of=/dev/null hash=md5,sha1 log=HD_001.log progress=on progresscount=1000 hashwindow=1900M

    if [ $? -ne 0 ]
    then
        echo "cat Join Run Failed"
        exit 3
    fi

    check_hashes

    # TODO check piecewise hashes
}

function join_cat3()
{
    cleanup

    "$DC3DD" conv=sync,noerror if="$DEV" iflag=direct hash=md5,sha1 hashwindow=1900M log=HD_001.log split=1900M splitformat=111 of=hd_001 progress=on progresscount=1000 errors=group

    if [ $? -ne 0 ]
    then
        echo "Acquisition Run Failed"
        exit 3
    fi

    cat hd_001.* | "$DC3DD" of=/dev/null hash=md5,sha1 log=HD_001.log progress=on progresscount=1000 hashwindow=1900M

    if [ $? -ne 0 ]
    then
        echo "cat Join Run Failed"
        exit 3
    fi

    check_hashes

    # TODO check piecewise hashes
}

join_cat
join_cat2
join_cat3
cleanup

